@@ -3,7 +3,7 @@ package Schedule::Activity::Attribute;
 use strict;
 use warnings;
 
-our $VERSION='0.1.8';
+our $VERSION='0.1.9';
 
 my %types=(
 	int=>{
@@ -82,6 +82,15 @@ sub average {
 	return $$self{avg};
 }
 
+sub reset {
+	my ($self)=@_;
+	foreach my $tm (sort {$a<=>$b} keys %{$$self{log}}) { $$self{tmmax}=$tm; $$self{value}=$$self{log}{$tm}; last }
+	%{$$self{log}}=();
+	$$self{avg}=undef;
+	$$self{tmsum}=undef;
+	return $self;
+}
+
 sub dump {
 	my ($self)=@_;
 	my %res=(
