@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Schedule::Activity::Attribute;
 
-our $VERSION='0.1.8';
+our $VERSION='0.1.9';
 
 sub new {
 	my ($ref,%opt)=@_;
@@ -54,6 +54,12 @@ sub report {
 	return %res;
 }
 
+sub reset {
+	my ($self)=@_;
+	foreach my $attr (values %{$$self{attr}}) { $attr->reset() }
+	return $self;
+}
+
 sub push {
 	my ($self)=@_;
 	my %state;
